@@ -11,32 +11,29 @@
 //
 //
 
+#include "config.h"
 #include "DialogBase.h"
 
+#include <glib/gi18n.h>
+#include <glog/logging.h>
 #include <sys/stat.h>
 
-#include <gdk/gdkkeysyms.h>
-
-#include "iptux/AnalogFS.h"
-#include "iptux/Command.h"
-#include "iptux/HelpDialog.h"
-#include "iptux/LogSystem.h"
-#include "iptux/SendFile.h"
+#include "iptux-utils/output.h"
+#include "iptux-utils/utils.h"
+#include "iptux/UiCoreThread.h"
+#include "iptux/UiHelper.h"
 #include "iptux/callback.h"
-#include "iptux/deplib.h"
-#include "iptux/global.h"
-#include "iptux/output.h"
-#include "iptux/support.h"
-#include "iptux/utils.h"
+
+using namespace std;
 
 namespace iptux {
 
-DialogBase::DialogBase(GroupInfo *grp, ProgramData &progdt)
-    : progdt(progdt),
+DialogBase::DialogBase(Application* app, GroupInfo* grp)
+    : app(app),
+      progdt(app->getProgramData()),
       widset(NULL),
       mdlset(NULL),
       dtset(NULL),
-      accel(NULL),
       grpinf(grp),
       totalsendsize(0),
       timersend(0) {
@@ -44,7 +41,8 @@ DialogBase::DialogBase(GroupInfo *grp, ProgramData &progdt)
 }
 
 DialogBase::~DialogBase() {
-  if (timersend > 0) g_source_remove(timersend);
+  if (timersend > 0)
+    g_source_remove(timersend);
   ClearSublayerGeneral();
 }
 
@@ -55,30 +53,28 @@ void DialogBase::InitSublayerGeneral() {
   g_datalist_init(&widset);
   g_datalist_init(&mdlset);
   g_datalist_init(&dtset);
-  accel = gtk_accel_group_new();
 }
 
 /**
  * 清空底层数据.
  */
 void DialogBase::ClearSublayerGeneral() {
-  if (progdt.IsAutoCleanChatHistory()) {
+  if (progdt->IsAutoCleanChatHistory()) {
     ClearHistoryTextView();
   }
-  grpinf->dialog = NULL;
+  grpinf->clearDialog();
   g_datalist_clear(&widset);
   g_datalist_clear(&mdlset);
   g_datalist_clear(&dtset);
-  g_object_unref(accel);
 }
 
 /**
  * 清空聊天历史记录.
  */
 void DialogBase::ClearHistoryTextView() {
-  GtkWidget *widget;
-  GtkTextBuffer *buffer;
-  GtkTextTagTable *table;
+  GtkWidget* widget;
+  GtkTextBuffer* buffer;
+  GtkTextTagTable* table;
   GtkTextIter start, end;
   GSList *taglist, *tlist;
 
@@ -109,10 +105,10 @@ void DialogBase::ClearHistoryTextView() {
  * 滚动聊天历史记录区.
  */
 void DialogBase::ScrollHistoryTextview() {
-  GtkWidget *widget;
-  GtkTextBuffer *buffer;
+  GtkWidget* widget;
+  GtkTextBuffer* buffer;
   GtkTextIter end;
-  GtkTextMark *mark;
+  GtkTextMark* mark;
 
   widget = GTK_WIDGET(g_datalist_get_data(&widset, "history-textview-widget"));
   buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(widget));
@@ -135,7 +131,7 @@ void DialogBase::OnNewMessageComing() {
  * 在窗口打开并且没有设置为最顶端的窗口时候，用窗口在任务栏的闪动来提示用户
  */
 void DialogBase::NotifyUser() {
-  GtkWindow *window;
+  GtkWindow* window;
   window = GTK_WINDOW(g_datalist_get_data(&widset, "window-widget"));
   if (!gtk_window_has_toplevel_focus(window))
     gtk_window_set_urgency_hint(window, TRUE);
@@ -145,64 +141,61 @@ void DialogBase::NotifyUser() {
  * 添加附件.
  * @param list 文件链表
  */
-void DialogBase::AttachEnclosure(const GSList *list) {
+void DialogBase::AttachEnclosure(const GSList* list) {
   GtkWidget *widget, *pbar;
-  GtkTreeModel *model;
+  GtkTreeModel* model;
   GtkTreeIter iter;
-  GdkPixbuf *pixbuf, *rpixbuf, *dpixbuf;
+  const char* iconname;
   struct stat st;
   const GSList *tlist, *pallist;
-  AnalogFS afs;
   int64_t filesize;
   char *filename, *filepath, *progresstip;
-  FileInfo *file;
+  FileInfo* file;
   uint32_t filenum = 0;
 
-  /* 获取文件图标 */
-  rpixbuf = obtain_pixbuf_from_stock(GTK_STOCK_FILE);
-  dpixbuf = obtain_pixbuf_from_stock(GTK_STOCK_DIRECTORY);
-
   /* 插入附件树 */
   widget =
       GTK_WIDGET(g_datalist_get_data(&widset, "file-send-treeview-widget"));
   model = gtk_tree_view_get_model(GTK_TREE_VIEW(widget));
   tlist = list;
+  auto g_cthrd = app->getCoreThread();
   while (tlist) {
-    if (stat((const char *)tlist->data, &st) == -1 ||
+    if (stat((const char*)tlist->data, &st) == -1 ||
         !(S_ISREG(st.st_mode) || S_ISDIR(st.st_mode))) {
       tlist = g_slist_next(tlist);
       continue;
     }
     /* 获取文件类型图标 */
-    if (S_ISREG(st.st_mode))
-      pixbuf = rpixbuf;
-    else if (S_ISDIR(st.st_mode))
-      pixbuf = dpixbuf;
-    else
-      pixbuf = NULL;
-    filesize = afs.ftwsize((char *)tlist->data);
-    filename = ipmsg_get_filename_me((char *)tlist->data, &filepath);
+    if (S_ISREG(st.st_mode)) {
+      iconname = "text-x-generic-symbolic";
+    } else if (S_ISDIR(st.st_mode)) {
+      iconname = "folder-symbolic";
+    } else {
+      iconname = NULL;
+    }
+    filesize = utils::fileOrDirectorySize((char*)tlist->data);
+    filename = ipmsg_get_filename_me((char*)tlist->data, &filepath);
     pallist = GetSelPal();
     while (pallist) {
       file = new FileInfo;
       file->fileid = g_cthrd->PrnQuote()++;
       /* file->packetn = 0;//没必要设置此字段 */
       file->fileattr =
-          S_ISREG(st.st_mode) ? IPMSG_FILE_REGULAR : IPMSG_FILE_DIR;
+          S_ISREG(st.st_mode) ? FileAttr::REGULAR : FileAttr::DIRECTORY;
       file->filesize = filesize;
-      file->filepath = g_strdup((char *)tlist->data);
+      file->filepath = g_strdup((char*)tlist->data);
       file->filectime = uint32_t(st.st_ctime);
       file->filenum = filenum;
-      file->fileown = (PalInfo *)(pallist->data);
+      file->fileown = g_cthrd->GetPal(((PalInfo*)(pallist->data))->GetKey());
       /* 加入文件信息到中心节点 */
       g_cthrd->Lock();
-      g_cthrd->AttachFileToPrivate(file);
+      g_cthrd->AddPrivateFile(PFileInfo(file));
       g_cthrd->Unlock();
       /* 添加数据 */
       gtk_list_store_append(GTK_LIST_STORE(model), &iter);
-      gtk_list_store_set(GTK_LIST_STORE(model), &iter, 0, pixbuf, 1, filename,
+      gtk_list_store_set(GTK_LIST_STORE(model), &iter, 0, iconname, 1, filename,
                          2, numeric_to_size(filesize), 3, tlist->data, 4, file,
-                         5, file->fileown->name, -1);
+                         5, file->fileown->getName().c_str(), -1);
       pallist = g_slist_next(pallist);
     }
     filenum++;
@@ -211,14 +204,12 @@ void DialogBase::AttachEnclosure(const GSList *list) {
   }
   //计算待发送文件总计大小
   totalsendsize = 0;
-  gtk_tree_model_get_iter_first(model, &iter);
-  do {  //遍历待发送model
-    gtk_tree_model_get(model, &iter, 4, &file, -1);
-    totalsendsize += file->filesize;
-  } while (gtk_tree_model_iter_next(model, &iter));
-  /* 释放文件图标 */
-  if (rpixbuf) g_object_unref(rpixbuf);
-  if (dpixbuf) g_object_unref(dpixbuf);
+  if (gtk_tree_model_get_iter_first(model, &iter)) {
+    do {  //遍历待发送model
+      gtk_tree_model_get(model, &iter, 4, &file, -1);
+      totalsendsize += file->filesize;
+    } while (gtk_tree_model_iter_next(model, &iter));
+  }
 
   pbar =
       GTK_WIDGET(g_datalist_get_data(&widset, "file-send-progress-bar-widget"));
@@ -231,7 +222,7 @@ void DialogBase::AttachEnclosure(const GSList *list) {
 /*
  * 主窗口的信号连接
  */
-void DialogBase::MainWindowSignalSetup(GtkWidget *window) {
+void DialogBase::MainWindowSignalSetup(GtkWindow* window) {
   g_object_set_data(G_OBJECT(window), "session-class", this);
   g_signal_connect_swapped(window, "destroy", G_CALLBACK(DialogDestory), this);
   g_signal_connect_swapped(window, "drag-data-received",
@@ -245,14 +236,14 @@ void DialogBase::MainWindowSignalSetup(GtkWidget *window) {
  * 创建消息输入区域.
  * @return 主窗体
  */
-GtkWidget *DialogBase::CreateInputArea() {
+GtkWidget* DialogBase::CreateInputArea() {
   GtkWidget *frame, *box, *sw;
   GtkWidget *hbb, *button;
   GtkWidget *widget, *window;
 
   frame = gtk_frame_new(NULL);
   gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_ETCHED_IN);
-  box = gtk_vbox_new(FALSE, 0);
+  box = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
   gtk_container_add(GTK_CONTAINER(frame), box);
 
   /* 接受输入 */
@@ -262,7 +253,9 @@ GtkWidget *DialogBase::CreateInputArea() {
   gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(sw),
                                       GTK_SHADOW_ETCHED_IN);
   gtk_box_pack_start(GTK_BOX(box), sw, TRUE, TRUE, 0);
-  widget = gtk_text_view_new();
+
+  widget = gtk_text_view_new_with_buffer(grpinf->getInputBuffer());
+  inputTextviewWidget = GTK_TEXT_VIEW(widget);
   gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(widget), GTK_WRAP_WORD);
   gtk_drag_dest_add_uri_targets(widget);
   gtk_container_add(GTK_CONTAINER(sw), widget);
@@ -272,7 +265,7 @@ GtkWidget *DialogBase::CreateInputArea() {
 
   /* 功能按钮 */
   window = GTK_WIDGET(g_datalist_get_data(&widset, "window-widget"));
-  hbb = gtk_hbutton_box_new();
+  hbb = gtk_button_box_new(GTK_ORIENTATION_HORIZONTAL);
   gtk_button_box_set_layout(GTK_BUTTON_BOX(hbb), GTK_BUTTONBOX_END);
   gtk_box_pack_start(GTK_BOX(box), hbb, FALSE, FALSE, 0);
   button = gtk_button_new_with_label(_("Close"));
@@ -280,12 +273,8 @@ GtkWidget *DialogBase::CreateInputArea() {
   g_signal_connect_swapped(button, "clicked", G_CALLBACK(gtk_widget_destroy),
                            window);
   button = gtk_button_new_with_label(_("Send"));
-  gtk_widget_add_accelerator(
-      button, "clicked", accel, GDK_KEY_Return,
-      progdt.IsEnterSendMessage() ? GdkModifierType(0) : GDK_CONTROL_MASK,
-      GTK_ACCEL_VISIBLE);
   gtk_box_pack_end(GTK_BOX(hbb), button, FALSE, FALSE, 0);
-  g_signal_connect_swapped(button, "clicked", G_CALLBACK(SendMessage), this);
+  gtk_actionable_set_action_name(GTK_ACTIONABLE(button), "win.send_message");
 
   return frame;
 }
@@ -294,9 +283,9 @@ GtkWidget *DialogBase::CreateInputArea() {
  * 创建聊天历史记录区域.
  * @return 主窗体.
  */
-GtkWidget *DialogBase::CreateHistoryArea() {
+GtkWidget* DialogBase::CreateHistoryArea() {
   GtkWidget *frame, *sw;
-  GtkWidget *widget;
+  GtkWidget* widget;
 
   frame = gtk_frame_new(_("Chat History"));
   gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_ETCHED_IN);
@@ -328,99 +317,29 @@ GtkWidget *DialogBase::CreateHistoryArea() {
   return frame;
 }
 
-/**
- * 创建文件菜单.
- * @return 菜单
- */
-GtkWidget *DialogBase::CreateFileMenu() {
-  GtkWidget *menushell, *window, *treeview;
-  GtkWidget *menu, *menuitem;
-
-  window = GTK_WIDGET(g_datalist_get_data(&widset, "window-widget"));
-  menushell = gtk_menu_item_new_with_mnemonic(_("_File"));
-  menu = gtk_menu_new();
-  gtk_menu_item_set_submenu(GTK_MENU_ITEM(menushell), menu);
-
-  menuitem = gtk_menu_item_new_with_label(_("Attach File"));
-  gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
-  g_signal_connect_swapped(menuitem, "activate", G_CALLBACK(AttachRegular),
-                           this);
-  gtk_widget_add_accelerator(menuitem, "activate", accel, GDK_KEY_S,
-                             GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
-
-  menuitem = gtk_menu_item_new_with_label(_("Attach Folder"));
-  gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
-  g_signal_connect_swapped(menuitem, "activate", G_CALLBACK(AttachFolder),
-                           this);
-  gtk_widget_add_accelerator(menuitem, "activate", accel, GDK_KEY_D,
-                             GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
-
-  treeview =
-      GTK_WIDGET(g_datalist_get_data(&widset, "file-send-treeview-widget"));
-  menuitem = gtk_menu_item_new_with_label(_("Remove Selected"));
-  gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
-  g_signal_connect_swapped(menuitem, "activate",
-                           G_CALLBACK(RemoveSelectedFromTree), treeview);
-  gtk_widget_add_accelerator(menuitem, "activate", accel, GDK_KEY_R,
-                             GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
-
-  menuitem = gtk_tearoff_menu_item_new();
-  gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
-
-  menuitem = gtk_menu_item_new_with_label(_("Close"));
-  gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
-  g_signal_connect_swapped(menuitem, "activate", G_CALLBACK(gtk_widget_destroy),
-                           window);
-  gtk_widget_add_accelerator(menuitem, "activate", accel, GDK_KEY_W,
-                             GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
-
-  g_datalist_set_data(&widset, "file-menu", menu);
-  return menushell;
-}
-
-/**
- * 创建帮助菜单.
- * @return 菜单
- */
-GtkWidget *DialogBase::CreateHelpMenu() {
-  GtkWidget *menushell;
-  GtkWidget *menu, *menuitem;
-
-  menushell = gtk_menu_item_new_with_mnemonic(_("_Help"));
-  menu = gtk_menu_new();
-  gtk_menu_item_set_submenu(GTK_MENU_ITEM(menushell), menu);
-
-  menuitem = gtk_image_menu_item_new_from_stock(GTK_STOCK_ABOUT, accel);
-  gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
-  g_signal_connect(menuitem, "activate", G_CALLBACK(HelpDialog::AboutEntry),
-                   NULL);
-
-  return menushell;
-}
-
 /**
  * 选择附件.
  * @param fileattr 文件类型
  * @return 文件链表
  */
-GSList *DialogBase::PickEnclosure(uint32_t fileattr) {
-  GtkWidget *dialog, *parent;
+GSList* DialogBase::PickEnclosure(FileAttr fileattr) {
+  GtkWidget* dialog;
   GtkFileChooserAction action;
-  const char *title;
-  GSList *list;
+  const char* title;
+  GSList* list;
 
-  if (GET_MODE(fileattr) == IPMSG_FILE_REGULAR) {
+  if (fileattr == FileAttr::REGULAR) {
     action = GTK_FILE_CHOOSER_ACTION_OPEN;
     title = _("Choose enclosure files");
-  } else {
+  } else if (fileattr == FileAttr::DIRECTORY) {
     action = GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER;
     title = _("Choose enclosure folders");
+  } else {
+    CHECK(false);
   }
-  parent = GTK_WIDGET(g_datalist_get_data(&widset, "dialog-widget"));
-
-  dialog = gtk_file_chooser_dialog_new(
-      title, GTK_WINDOW(parent), action, GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
-      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, NULL);
+  dialog = gtk_file_chooser_dialog_new(title, GTK_WINDOW(getWindow()), action,
+                                       _("_Open"), GTK_RESPONSE_ACCEPT,
+                                       _("_Cancel"), GTK_RESPONSE_CANCEL, NULL);
   gtk_dialog_set_default_response(GTK_DIALOG(dialog), GTK_RESPONSE_ACCEPT);
   gtk_file_chooser_set_local_only(GTK_FILE_CHOOSER(dialog), FALSE);
   gtk_file_chooser_set_select_multiple(GTK_FILE_CHOOSER(dialog), TRUE);
@@ -446,28 +365,30 @@ GSList *DialogBase::PickEnclosure(uint32_t fileattr) {
  * @return 是否发送数据
  */
 bool DialogBase::SendEnclosureMsg() {
-  GtkWidget *treeview;
-  GtkTreeModel *model;
+  GtkWidget* treeview;
+  GtkTreeModel* model;
   GtkTreeIter iter;
-  GSList *list;
-  gchar *filepath;
-  FileInfo *file;
+  gchar* filepath;
+  FileInfo* file;
 
   treeview =
       GTK_WIDGET(g_datalist_get_data(&widset, "file-send-treeview-widget"));
   model = gtk_tree_view_get_model(GTK_TREE_VIEW(treeview));
-  if (!gtk_tree_model_get_iter_first(model, &iter)) return false;
+  if (!gtk_tree_model_get_iter_first(model, &iter))
+    return false;
 
   /* 获取文件并发送 */
-  list = NULL;
+  vector<FileInfo*> files;
   do {
     gtk_tree_model_get(model, &iter, 3, &filepath, 4, &file, -1);
-    list = g_slist_append(list, file);
+    files.push_back(file);
   } while (gtk_tree_model_iter_next(model, &iter));
+  gtk_list_store_clear(GTK_LIST_STORE(model));
 
-  BroadcastEnclosureMsg(list);
-  g_slist_free(list);
-  timersend = gdk_threads_add_timeout(400, (GSourceFunc)UpdateFileSendUI, this);
+  BroadcastEnclosureMsg(files);
+  if (!timersend) {
+    timersend = g_timeout_add(400, (GSourceFunc)UpdateFileSendUI, this);
+  }
   return true;
 }
 
@@ -475,31 +396,25 @@ bool DialogBase::SendEnclosureMsg() {
  * 回馈消息.
  * @param msg 消息
  */
-void DialogBase::FeedbackMsg(const gchar *msg) {
-  MsgPara para;
-  ChipData *chip;
+void DialogBase::FeedbackMsg(const gchar* msg) {
+  MsgPara para(this->app->getMe());
+  para.stype = MessageSourceType::SELF;
+  para.btype = grpinf->getType();
 
-  /* 构建消息封装包 */
-  para.pal = NULL;
-  para.stype = MESSAGE_SOURCE_TYPE_SELF;
-  para.btype = grpinf->type;
-  chip = new ChipData;
-  chip->type = MESSAGE_CONTENT_TYPE_STRING;
-  chip->data = g_strdup(msg);
-  para.dtlist = g_slist_append(NULL, chip);
-
-  /* 交给某人处理吧 */
-  g_cthrd->InsertMsgToGroupInfoItem(grpinf, &para);
+  ChipData chip(msg);
+  para.dtlist.push_back(std::move(chip));
+  grpinf->addMsgPara(para);
 }
 
 /**
  * 添加常规文件附件.
  * @param dlgpr 对话框类
  */
-void DialogBase::AttachRegular(DialogBase *dlgpr) {
-  GSList *list;
+void DialogBase::AttachRegular(DialogBase* dlgpr) {
+  GSList* list;
 
-  if (!(list = dlgpr->PickEnclosure(IPMSG_FILE_REGULAR))) return;
+  if (!(list = dlgpr->PickEnclosure(FileAttr::REGULAR)))
+    return;
   dlgpr->AttachEnclosure(list);
   g_slist_foreach(list, GFunc(g_free), NULL);
   g_slist_free(list);
@@ -509,28 +424,21 @@ void DialogBase::AttachRegular(DialogBase *dlgpr) {
  * 添加目录文件附件.
  * @param dlgpr 对话框类
  */
-void DialogBase::AttachFolder(DialogBase *dlgpr) {
-  GSList *list;
+void DialogBase::AttachFolder(DialogBase* dlgpr) {
+  GSList* list;
 
-  if (!(list = dlgpr->PickEnclosure(IPMSG_FILE_DIR))) return;
+  if (!(list = dlgpr->PickEnclosure(FileAttr::DIRECTORY)))
+    return;
   dlgpr->AttachEnclosure(list);
   g_slist_foreach(list, GFunc(g_free), NULL);
   g_slist_free(list);
 }
 
-/**
- * 清空聊天历史记录缓冲区.
- * @param dlgpr 对话框类
- */
-void DialogBase::ClearHistoryBuffer(DialogBase *dlgpr) {
-  dlgpr->ClearHistoryTextView();
-}
-
 /**
  * 发送消息.
  * @param dlgpr 对话框类
  */
-void DialogBase::SendMessage(DialogBase *dlgpr) {
+void DialogBase::SendMessage(DialogBase* dlgpr) {
   dlgpr->SendEnclosureMsg();
   dlgpr->SendTextMsg();
   dlgpr->ScrollHistoryTextview();
@@ -539,14 +447,19 @@ void DialogBase::SendMessage(DialogBase *dlgpr) {
  * 对话框被摧毁的回调函数
  * @param dialog
  */
-void DialogBase::DialogDestory(DialogBase *dialog) { delete dialog; }
+void DialogBase::DialogDestory(DialogBase* dialog) {
+  delete dialog;
+}
 
 /**
  * 清除提示,这个提示只是窗口闪动的提示
  */
-gboolean DialogBase::ClearNotify(GtkWidget *window, GdkEventConfigure *event) {
+gboolean DialogBase::ClearNotify(GtkWidget* window, GdkEventConfigure*) {
   if (gtk_window_get_urgency_hint(GTK_WINDOW(window)))
     gtk_window_set_urgency_hint(GTK_WINDOW(window), FALSE);
+  DialogBase* self =
+      (DialogBase*)g_object_get_data(G_OBJECT(window), "session-class");
+  self->grpinf->readAllMsg();
   return FALSE;
 }
 
@@ -561,11 +474,14 @@ gboolean DialogBase::ClearNotify(GtkWidget *window, GdkEventConfigure *event) {
  * GtkTargetList
  * @param time the timestamp at which the data was received
  */
-void DialogBase::DragDataReceived(DialogBase *dlgpr, GdkDragContext *context,
-                                  gint x, gint y, GtkSelectionData *data,
-                                  guint info, guint time) {
-  GtkWidget *widget;
-  GSList *list;
+void DialogBase::DragDataReceived(DialogBase* dlgpr,
+                                  GdkDragContext* context,
+                                  gint,
+                                  gint,
+                                  GtkSelectionData* data,
+                                  guint,
+                                  guint time) {
+  GSList* list;
 
   if (!ValidateDragData(data, context, time)) {
     return;
@@ -575,10 +491,6 @@ void DialogBase::DragDataReceived(DialogBase *dlgpr, GdkDragContext *context,
   dlgpr->AttachEnclosure(list);
   g_slist_foreach(list, GFunc(g_free), NULL);
   g_slist_free(list);
-  widget =
-      GTK_WIDGET(g_datalist_get_data(&dlgpr->widset, "enclosure-frame-widget"));
-  gtk_widget_show(widget);
-
   gtk_drag_finish(context, TRUE, FALSE, time);
 }
 
@@ -589,9 +501,9 @@ void DialogBase::DragDataReceived(DialogBase *dlgpr, GdkDragContext *context,
  * @param dtset data set
  * @return Gtk+库所需
  */
-gboolean DialogBase::WindowConfigureEvent(GtkWidget *window,
-                                          GdkEventConfigure *event,
-                                          GData **dtset) {
+gboolean DialogBase::WindowConfigureEvent(GtkWidget*,
+                                          GdkEventConfigure* event,
+                                          GData** dtset) {
   g_datalist_set_data(dtset, "window-width", GINT_TO_POINTER(event->width));
   g_datalist_set_data(dtset, "window-height", GINT_TO_POINTER(event->height));
 
@@ -604,12 +516,13 @@ gboolean DialogBase::WindowConfigureEvent(GtkWidget *window,
  * @param pspec he GParamSpec of the property which changed
  * @param dtset data set
  */
-void DialogBase::PanedDivideChanged(GtkWidget *paned, GParamSpec *pspec,
-                                    GData **dtset) {
-  const gchar *identify;
+void DialogBase::PanedDivideChanged(GtkWidget* paned,
+                                    GParamSpec* /*pspec*/,
+                                    GData** dtset) {
+  const gchar* identify;
   gint position;
 
-  identify = (const gchar *)g_object_get_data(G_OBJECT(paned), "position-name");
+  identify = (const gchar*)g_object_get_data(G_OBJECT(paned), "position-name");
   position = gtk_paned_get_position(GTK_PANED(paned));
   g_datalist_set_data(dtset, identify, GINT_TO_POINTER(position));
 }
@@ -618,22 +531,23 @@ void DialogBase::PanedDivideChanged(GtkWidget *paned, GParamSpec *pspec,
  *删除选定附件.
  * @param dlgpr 对话框类
  */
-void DialogBase::RemoveSelectedFromTree(GtkWidget *widget) {
-  GList *list;
-  GtkTreeSelection *TreeSel;
-  GtkTreePath *path;
-  GtkTreeModel *model;
-  gchar *str_data;
+void DialogBase::RemoveSelectedFromTree(GtkWidget* widget) {
+  GList* list;
+  GtkTreeSelection* TreeSel;
+  GtkTreePath* path;
+  GtkTreeModel* model;
+  gchar* str_data;
   gboolean valid = 0;
   GtkTreeIter iter;
 
   model = gtk_tree_view_get_model(GTK_TREE_VIEW(widget));
   TreeSel = gtk_tree_view_get_selection(GTK_TREE_VIEW(widget));
   list = gtk_tree_selection_get_selected_rows(TreeSel, NULL);
-  if (!list) return;
+  if (!list)
+    return;
   while (list) {
     gtk_tree_model_get_iter(GTK_TREE_MODEL(model), &iter,
-                            (GtkTreePath *)g_list_nth(list, 0)->data);
+                            (GtkTreePath*)g_list_nth(list, 0)->data);
     gtk_list_store_set(GTK_LIST_STORE(model), &iter, 2, "delete", -1);
     list = g_list_next(list);
   }
@@ -655,22 +569,19 @@ void DialogBase::RemoveSelectedFromTree(GtkWidget *widget) {
  * @param widget TreeView
  * @param event 事件
  */
-gint DialogBase::EncosureTreePopup(GtkWidget *treeview, GdkEvent *event) {
+gint DialogBase::EnclosureTreePopup(DialogBase* self, GdkEvent* event) {
   GtkWidget *menu, *menuitem;
-  GdkEventButton *event_button;
 
   menu = gtk_menu_new();
   menuitem = gtk_menu_item_new_with_label(_("Remove Selected"));
   g_signal_connect_swapped(menuitem, "activate",
-                           G_CALLBACK(RemoveSelectedEnclosure), treeview);
+                           G_CALLBACK(RemoveSelectedEnclosure), self);
   gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
 
   if (event->type == GDK_BUTTON_PRESS) {
-    event_button = (GdkEventButton *)event;
-    if (event_button->button == 3) {
-      gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL,
-                     event_button->button, event_button->time);
+    if (gdk_event_triggers_context_menu(event)) {
       gtk_widget_show(menuitem);
+      gtk_menu_popup_at_pointer(GTK_MENU(menu), nullptr);
       return TRUE;
     }
   }
@@ -680,31 +591,32 @@ gint DialogBase::EncosureTreePopup(GtkWidget *treeview, GdkEvent *event) {
  *从显示附件的TreeView删除选定行.
  * @param widget TreeView
  */
-void DialogBase::RemoveSelectedEnclosure(GtkWidget *widget) {
-  GtkTreeModel *model;
-  GtkTreeSelection *TreeSel;
+void DialogBase::RemoveSelectedEnclosure(DialogBase* self) {
+  GtkTreeModel* model;
+  GtkTreeSelection* TreeSel;
   GtkTreeIter iter;
-  FileInfo *file;
-  DialogBase *dlg;
-  GList *list;
+  FileInfo* file;
+  GList* list;
+  auto widget = self->fileSendTree;
+  auto dlg = self;
 
-  dlg = (DialogBase *)(g_object_get_data(G_OBJECT(widget), "dialog"));
   model = gtk_tree_view_get_model(GTK_TREE_VIEW(widget));
   //从中心结点删除
   TreeSel = gtk_tree_view_get_selection(GTK_TREE_VIEW(widget));
   list = gtk_tree_selection_get_selected_rows(TreeSel, NULL);
-  if (!list) return;
+  if (!list)
+    return;
   while (list) {
     gtk_tree_model_get_iter(GTK_TREE_MODEL(model), &iter,
-                            (GtkTreePath *)g_list_nth(list, 0)->data);
+                            (GtkTreePath*)g_list_nth(list, 0)->data);
     gtk_tree_model_get(model, &iter, 4, &file, -1);
     dlg->totalsendsize -= file->filesize;
-    g_cthrd->DelFileFromPrivate(file->fileid);
+    self->app->getCoreThread()->DelPrivateFile(file->fileid);
     list = g_list_next(list);
   }
   g_list_free(list);
   //从列表中删除
-  RemoveSelectedFromTree(widget);
+  RemoveSelectedFromTree(GTK_WIDGET(widget));
   //重新计算待发送文件大小
   dlg->UpdateFileSendUI(dlg);
 }
@@ -713,16 +625,16 @@ void DialogBase::RemoveSelectedEnclosure(GtkWidget *widget) {
  * 创建文件发送区域.
  * @return 主窗体
  */
-GtkWidget *DialogBase::CreateFileSendArea() {
+GtkWidget* DialogBase::CreateFileSendArea() {
   GtkWidget *frame, *hbox, *vbox, *button, *pbar, *sw, *treeview;
-  GtkTreeModel *model;
+  GtkTreeModel* model;
   frame = gtk_frame_new(_("File to send."));
   g_datalist_set_data(&widset, "file-send-frame-widget", frame);
   gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_ETCHED_IN);
   pbar = gtk_progress_bar_new();
   g_datalist_set_data(&widset, "file-send-progress-bar-widget", pbar);
   gtk_progress_bar_set_text(GTK_PROGRESS_BAR(pbar), _("Sending progress."));
-  hbox = gtk_hbox_new(FALSE, 1);
+  hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 1);
   gtk_box_pack_start(GTK_BOX(hbox), pbar, TRUE, TRUE, 0);
   button = gtk_button_new_with_label(_("Dirs"));
   gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, TRUE, 0);
@@ -732,8 +644,9 @@ GtkWidget *DialogBase::CreateFileSendArea() {
   g_signal_connect_swapped(button, "clicked", G_CALLBACK(AttachRegular), this);
   button = gtk_button_new_with_label(_("Detail"));
   gtk_box_pack_end(GTK_BOX(hbox), button, FALSE, TRUE, 0);
-  g_signal_connect_swapped(button, "clicked", G_CALLBACK(OpenTransDlg), NULL);
-  vbox = gtk_vbox_new(FALSE, 0);
+  gtk_actionable_set_action_name(GTK_ACTIONABLE(button),
+                                 "app.tools.transmission");
+  vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
   gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
   sw = gtk_scrolled_window_new(NULL, NULL);
   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw), GTK_POLICY_AUTOMATIC,
@@ -741,6 +654,7 @@ GtkWidget *DialogBase::CreateFileSendArea() {
   gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(sw),
                                       GTK_SHADOW_ETCHED_IN);
   model = CreateFileSendModel();
+  fileSendModel = GTK_LIST_STORE(model);
   treeview = CreateFileSendTree(model);
   g_datalist_set_data_full(&mdlset, "enclosure-model", model,
                            GDestroyNotify(g_object_unref));
@@ -758,26 +672,27 @@ GtkWidget *DialogBase::CreateFileSendArea() {
  * @param model FileSend-model
  * @return 待发送文件树
  */
-GtkWidget *DialogBase::CreateFileSendTree(GtkTreeModel *model) {
-  GtkWidget *view;
-  GtkTreeSelection *selection;
-  GtkCellRenderer *cell;
-  GtkTreeViewColumn *column;
+GtkWidget* DialogBase::CreateFileSendTree(GtkTreeModel* model) {
+  GtkWidget* view;
+  GtkTreeSelection* selection;
+  GtkCellRenderer* cell;
+  GtkTreeViewColumn* column;
 
   view = gtk_tree_view_new_with_model(model);
+  this->fileSendTree = GTK_TREE_VIEW(view);
   gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(view), TRUE);
   selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(view));
   gtk_tree_selection_set_mode(selection, GTK_SELECTION_MULTIPLE);
 
   cell = gtk_cell_renderer_pixbuf_new();
   column =
-      gtk_tree_view_column_new_with_attributes("", cell, "pixbuf", 0, NULL);
+      gtk_tree_view_column_new_with_attributes("", cell, "icon-name", 0, NULL);
   gtk_tree_view_column_set_resizable(column, TRUE);
   gtk_tree_view_append_column(GTK_TREE_VIEW(view), column);
 
-  if (grpinf->type != GROUP_BELONG_TYPE_REGULAR) {
+  if (grpinf->getType() != GROUP_BELONG_TYPE_REGULAR) {
     cell = gtk_cell_renderer_text_new();
-    column = gtk_tree_view_column_new_with_attributes(_("PeelName"), cell,
+    column = gtk_tree_view_column_new_with_attributes(_("PeerName"), cell,
                                                       "text", 5, NULL);
     gtk_tree_view_column_set_resizable(column, TRUE);
     gtk_tree_view_append_column(GTK_TREE_VIEW(view), column);
@@ -802,7 +717,7 @@ GtkWidget *DialogBase::CreateFileSendTree(GtkTreeModel *model) {
   gtk_tree_view_append_column(GTK_TREE_VIEW(view), column);
 
   g_signal_connect_swapped(GTK_WIDGET(view), "button_press_event",
-                           G_CALLBACK(EncosureTreePopup), view);
+                           G_CALLBACK(EnclosureTreePopup), this);
   return view;
 }
 
@@ -812,10 +727,10 @@ GtkWidget *DialogBase::CreateFileSendTree(GtkTreeModel *model) {
  * 0:图标 1:文件名 2:大小(string) 3:全文件名 4:文件信息(指针) 5:接收者
  * 没有专门加删除标记，用第2列作删除标记，(某行反正要删除，改就改了)
  */
-GtkTreeModel *DialogBase::CreateFileSendModel() {
-  GtkListStore *model;
+GtkTreeModel* DialogBase::CreateFileSendModel() {
+  GtkListStore* model;
 
-  model = gtk_list_store_new(6, GDK_TYPE_PIXBUF, G_TYPE_STRING, G_TYPE_STRING,
+  model = gtk_list_store_new(6, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING,
                              G_TYPE_STRING, G_TYPE_POINTER, G_TYPE_STRING);
 
   return GTK_TREE_MODEL(model);
@@ -827,23 +742,16 @@ GtkTreeModel *DialogBase::CreateFileSendModel() {
  * 让传输聊天窗口从传输状态窗口去取数据，而没有让文件数据发送类把数据传送到聊天窗口，
  * 这是因为考虑数据要发到本窗口，会存在窗口未打开或群聊状态等不确定因素,处理过程太复杂
  */
-gboolean DialogBase::UpdateFileSendUI(DialogBase *dlggrp) {
-  GtkTreeModel *model;
+gboolean DialogBase::UpdateFileSendUI(DialogBase* dlggrp) {
+  GtkTreeModel* model;
   GtkTreeIter iter;
-  char progresstip[MAX_BUFLEN];
-  GtkTreeView *treeview;
-  GtkWidget *pbar;
+  string progresstip;
+  GtkTreeView* treeview;
+  GtkWidget* pbar;
   float progress;
   int64_t sentsize;
-  GtkIconTheme *theme;
-  GdkPixbuf *pixbuf;
-  const char *statusfile;
-  FileInfo *file;
+  FileInfo* file;
 
-  theme = gtk_icon_theme_get_default();
-  statusfile = "tip-finish";
-  pixbuf = gtk_icon_theme_load_icon(theme, statusfile, MAX_ICONSIZE,
-                                    GtkIconLookupFlags(0), NULL);
   treeview = GTK_TREE_VIEW(
       g_datalist_get_data(&(dlggrp->widset), "file-send-treeview-widget"));
   model = gtk_tree_view_get_model(GTK_TREE_VIEW(treeview));
@@ -851,8 +759,9 @@ gboolean DialogBase::UpdateFileSendUI(DialogBase *dlggrp) {
   if (gtk_tree_model_get_iter_first(model, &iter)) {
     do {  //遍历待发送model
       gtk_tree_model_get(model, &iter, 4, &file, -1);
-      if (pixbuf && (file->finishedsize == file->filesize))
-        gtk_list_store_set(GTK_LIST_STORE(model), &iter, 0, pixbuf, -1);
+      if (file->finishedsize == file->filesize) {
+        gtk_list_store_set(GTK_LIST_STORE(model), &iter, 0, "tip-finish", -1);
+      }
       sentsize += file->finishedsize;
     } while (gtk_tree_model_iter_next(model, &iter));
   }
@@ -863,25 +772,25 @@ gboolean DialogBase::UpdateFileSendUI(DialogBase *dlggrp) {
 
   if (dlggrp->totalsendsize == 0) {
     progress = 0;
-    snprintf(progresstip, MAX_BUFLEN, "%s", _("Sending Progress."));
+    progresstip = _("Sending Progress.");
   } else {
     progress = percent(sentsize, dlggrp->totalsendsize) / 100;
-    snprintf(progresstip, MAX_BUFLEN, _("%s Of %s Sent."),
-             numeric_to_size(sentsize), numeric_to_size(dlggrp->totalsendsize));
+    progresstip = stringFormat(_("%s of %s Sent."), numeric_to_size(sentsize),
+                               numeric_to_size(dlggrp->totalsendsize));
   }
   if (progress == 1) {
     g_source_remove(dlggrp->timersend);
+    dlggrp->timersend = 0;
     gtk_list_store_clear(GTK_LIST_STORE(model));
-    snprintf(progresstip, MAX_BUFLEN, "%s", _("Mission Completed!"));
+    progresstip = _("Mission Completed!");
   }
   gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(pbar), progress);
-  gtk_progress_bar_set_text(GTK_PROGRESS_BAR(pbar), _(progresstip));
+  gtk_progress_bar_set_text(GTK_PROGRESS_BAR(pbar), progresstip.c_str());
   return TRUE;
 }
-/**
- * 打开文件传输窗口.
- * @param dlgpr 对话框类
- */
-void DialogBase::OpenTransDlg(DialogBase *dlgpr) { g_mwin->OpenTransWindow(); }
 
-}  // namespace iptux
\ No newline at end of file
+GtkTextBuffer* DialogBase::getInputBuffer() {
+  return grpinf->getInputBuffer();
+}
+
+}  // namespace iptux
