@@ -9,18 +9,21 @@
 // Copyright: See COPYING file that comes with this distribution
 //
 //
+#include "config.h"
 #include "DialogGroup.h"
 
-#include "iptux/Command.h"
+#include <glib/gi18n.h>
+#include <glog/logging.h>
+
+#include "iptux-core/Const.h"
+#include "iptux-utils/output.h"
+#include "iptux-utils/utils.h"
 #include "iptux/DialogPeer.h"
-#include "iptux/HelpDialog.h"
-#include "iptux/SendFile.h"
+#include "iptux/UiCoreThread.h"
+#include "iptux/UiHelper.h"
 #include "iptux/callback.h"
-#include "iptux/deplib.h"
-#include "iptux/global.h"
-#include "iptux/output.h"
-#include "iptux/support.h"
-#include "iptux/utils.h"
+
+using namespace std;
 
 namespace iptux {
 
@@ -28,28 +31,32 @@ namespace iptux {
  * 类构造函数.
  * @param grp 群组信息
  */
-DialogGroup::DialogGroup(GroupInfo *grp, IptuxConfig &config,
-                         ProgramData &progdt)
-    : DialogBase(grp, progdt), config(config) {
+DialogGroup::DialogGroup(Application* app, GroupInfo* grp)
+    : DialogBase(CHECK_NOTNULL(app), CHECK_NOTNULL(grp)),
+      config(app->getConfig()) {
   InitSublayerSpecify();
 }
 
 /**
  * 类析构函数.
  */
-DialogGroup::~DialogGroup() { SaveUILayout(); }
+DialogGroup::~DialogGroup() {
+  SaveUILayout();
+}
 
 /**
  * 群组对话框入口.
  * @param grpinf 群组信息
  */
-void DialogGroup::GroupDialogEntry(IptuxConfig &config, GroupInfo *grpinf,
-                                   ProgramData &progdt) {
-  DialogGroup *dlggrp;
+DialogGroup* DialogGroup::GroupDialogEntry(Application* app,
+                                           GroupInfo* grpinf) {
+  CHECK_NOTNULL(grpinf);
+  CHECK_NE(grpinf->getType(), GROUP_BELONG_TYPE_REGULAR);
+  DialogGroup* dlggrp;
   GtkWidget *window, *widget;
 
-  dlggrp = new DialogGroup(grpinf, config, progdt);
-  window = dlggrp->CreateMainWindow();
+  dlggrp = new DialogGroup(app, grpinf);
+  window = GTK_WIDGET(dlggrp->CreateMainWindow());
   gtk_container_add(GTK_CONTAINER(window), dlggrp->CreateAllArea());
   gtk_widget_show_all(window);
 
@@ -57,29 +64,21 @@ void DialogGroup::GroupDialogEntry(IptuxConfig &config, GroupInfo *grpinf,
   widget =
       GTK_WIDGET(g_datalist_get_data(&dlggrp->widset, "input-textview-widget"));
   gtk_widget_grab_focus(widget);
-  /* 从消息队列中移除 */
-  g_cthrd->Lock();
-  if (g_cthrd->MsglineContainItem(grpinf)) {
-    g_mwin->MakeItemBlinking(grpinf, FALSE);
-    g_cthrd->PopItemFromMsgline(grpinf);
-  }
-  g_cthrd->Unlock();
-
-  /* delete dlggrp;//请不要这样做，此类将会在窗口被摧毁后自动释放 */
+  return dlggrp;
 }
 
 /**
  * 更新群组成员树(member-tree)指定项.
  * @param pal class PalInfo
  */
-void DialogGroup::UpdatePalData(PalInfo *pal) {
-  GtkIconTheme *theme;
-  GdkPixbuf *pixbuf;
-  GtkWidget *widget;
-  GtkTreeModel *model;
+void DialogGroup::UpdatePalData(PalInfo* pal) {
+  GtkIconTheme* theme;
+  GdkPixbuf* pixbuf;
+  GtkWidget* widget;
+  GtkTreeModel* model;
   GtkTreeIter iter;
   gpointer data;
-  gchar *file;
+  gchar* file;
 
   /* 查询项所在的位置，若没有则添加 */
   widget = GTK_WIDGET(g_datalist_get_data(&widset, "member-treeview-widget"));
@@ -88,7 +87,8 @@ void DialogGroup::UpdatePalData(PalInfo *pal) {
   if (gtk_tree_model_get_iter_first(model, &iter)) {
     do {
       gtk_tree_model_get(model, &iter, 3, &data, -1);
-      if (data == pal) break;
+      if (data == pal)
+        break;
     } while (gtk_tree_model_iter_next(model, &iter));
   }
   if (data != pal) {
@@ -102,21 +102,23 @@ void DialogGroup::UpdatePalData(PalInfo *pal) {
   pixbuf = gtk_icon_theme_load_icon(theme, file, MAX_ICONSIZE,
                                     GtkIconLookupFlags(0), NULL);
   g_free(file);
-  gtk_list_store_set(GTK_LIST_STORE(model), &iter, 1, pixbuf, 2, pal->name, -1);
-  if (pixbuf) g_object_unref(pixbuf);
+  gtk_list_store_set(GTK_LIST_STORE(model), &iter, 1, pixbuf, 2,
+                     pal->getName().c_str(), -1);
+  if (pixbuf)
+    g_object_unref(pixbuf);
 }
 
 /**
  * 插入项到群组成员树(member-tree).
  * @param pal class PalInfo
  */
-void DialogGroup::InsertPalData(PalInfo *pal) {
-  GtkIconTheme *theme;
-  GdkPixbuf *pixbuf;
-  GtkWidget *widget;
-  GtkTreeModel *model;
+void DialogGroup::InsertPalData(PalInfo* pal) {
+  GtkIconTheme* theme;
+  GdkPixbuf* pixbuf;
+  GtkWidget* widget;
+  GtkTreeModel* model;
   GtkTreeIter iter;
-  gchar *file;
+  gchar* file;
 
   theme = gtk_icon_theme_get_default();
   file = iptux_erase_filename_suffix(pal->iconfile);
@@ -127,17 +129,18 @@ void DialogGroup::InsertPalData(PalInfo *pal) {
   model = gtk_tree_view_get_model(GTK_TREE_VIEW(widget));
   gtk_list_store_append(GTK_LIST_STORE(model), &iter);
   gtk_list_store_set(GTK_LIST_STORE(model), &iter, 0, FALSE, 1, pixbuf, 2,
-                     pal->name, 3, pal, -1);
-  if (pixbuf) g_object_unref(pixbuf);
+                     pal->getName().c_str(), 3, pal, -1);
+  if (pixbuf)
+    g_object_unref(pixbuf);
 }
 
 /**
  * 从群组成员树(member-tree)删除指定项.
  * @param pal class PalInfo
  */
-void DialogGroup::DelPalData(PalInfo *pal) {
-  GtkWidget *widget;
-  GtkTreeModel *model;
+void DialogGroup::DelPalData(PalInfo* pal) {
+  GtkWidget* widget;
+  GtkTreeModel* model;
   GtkTreeIter iter;
   gpointer data;
 
@@ -158,8 +161,8 @@ void DialogGroup::DelPalData(PalInfo *pal) {
  * 清除本群组所有好友数据.
  */
 void DialogGroup::ClearAllPalData() {
-  GtkWidget *widget;
-  GtkTreeModel *model;
+  GtkWidget* widget;
+  GtkTreeModel* model;
 
   widget = GTK_WIDGET(g_datalist_get_data(&widset, "member-treeview-widget"));
   model = gtk_tree_view_get_model(GTK_TREE_VIEW(widget));
@@ -170,7 +173,7 @@ void DialogGroup::ClearAllPalData() {
  * 初始化底层数据.
  */
 void DialogGroup::InitSublayerSpecify() {
-  GtkTreeModel *model;
+  GtkTreeModel* model;
   model = CreateMemberModel();
   g_datalist_set_data_full(&mdlset, "member-model", model,
                            GDestroyNotify(g_object_unref));
@@ -181,120 +184,103 @@ void DialogGroup::InitSublayerSpecify() {
  * 写出对话框的UI布局数据.
  */
 void DialogGroup::SaveUILayout() {
-  config.SetInt("group_window_width",
-                GPOINTER_TO_INT(g_datalist_get_data(&dtset, "window-width")));
-  config.SetInt("group_window_height",
-                GPOINTER_TO_INT(g_datalist_get_data(&dtset, "window-height")));
-  config.SetInt("group_main_paned_divide", GPOINTER_TO_INT(g_datalist_get_data(
-                                               &dtset, "main-paned-divide")));
-  config.SetInt("group_historyinput_paned_divide",
-                GPOINTER_TO_INT(
-                    g_datalist_get_data(&dtset, "historyinput-paned-divide")));
-  config.SetInt("group_memberenclosure_paned_divide",
-                GPOINTER_TO_INT(g_datalist_get_data(
-                    &dtset, "memberenclosure-paned-divide")));
+  config->SetInt("group_window_width",
+                 GPOINTER_TO_INT(g_datalist_get_data(&dtset, "window-width")));
+  config->SetInt("group_window_height",
+                 GPOINTER_TO_INT(g_datalist_get_data(&dtset, "window-height")));
 }
 
 /**
  * 创建主窗口.
  * @return 窗口
  */
-GtkWidget *DialogGroup::CreateMainWindow() {
+GtkWindow* DialogGroup::CreateMainWindow() {
   char buf[MAX_BUFLEN];
-  GtkWidget *window;
-
-  window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
-  snprintf(buf, MAX_BUFLEN, _("Talk with the group %s"), grpinf->name);
+  window = GTK_APPLICATION_WINDOW(gtk_application_window_new(app->getApp()));
+  snprintf(buf, MAX_BUFLEN, _("Talk with the group %s"), grpinf->name.c_str());
   gtk_window_set_title(GTK_WINDOW(window), buf);
   gtk_window_set_default_size(GTK_WINDOW(window),
-                              config.GetInt("group_window_width", 500),
-                              config.GetInt("group_window_height", 350));
+                              config->GetInt("group_window_width", 500),
+                              config->GetInt("group_window_height", 350));
   gtk_window_set_position(GTK_WINDOW(window), GTK_WIN_POS_CENTER);
-  gtk_window_add_accel_group(GTK_WINDOW(window), accel);
   g_datalist_set_data(&widset, "window-widget", window);
-  widget_enable_dnd_uri(window);
-  grpinf->dialog = window;
+  widget_enable_dnd_uri(GTK_WIDGET(window));
+  grpinf->setDialogBase(this);
 
-  MainWindowSignalSetup(window);
+  MainWindowSignalSetup(GTK_WINDOW(window));
 
-  return window;
+  GActionEntry win_entries[] = {
+      makeActionEntry("clear_chat_history",
+                      G_ACTION_CALLBACK(onClearChatHistory)),
+      makeActionEntry("attach_file", G_ACTION_CALLBACK(onAttachFile)),
+      makeActionEntry("attach_folder", G_ACTION_CALLBACK(onAttachFolder)),
+      makeStateActionEntry("sort_type", G_ACTION_CALLBACK(onSortType), "s",
+                           "'ascending'"),
+      makeStateActionEntry("sort_by", G_ACTION_CALLBACK(onSortBy), "s",
+                           "'nickname'"),
+      makeActionEntry("send_message", G_ACTION_CALLBACK(onSendMessage)),
+  };
+  g_action_map_add_action_entries(G_ACTION_MAP(window), win_entries,
+                                  G_N_ELEMENTS(win_entries), this);
+  return GTK_WINDOW(window);
 }
 
 /**
  * 创建所有区域.
  * @return 主窗体
  */
-GtkWidget *DialogGroup::CreateAllArea() {
-  GtkWidget *box;
-  GtkWidget *hpaned, *vpaned;
-  gint position;
+GtkWidget* DialogGroup::CreateAllArea() {
+  GtkWidget* box;
 
-  box = gtk_vbox_new(FALSE, 0);
-
-  /* 加入菜单条 */
-  gtk_box_pack_start(GTK_BOX(box), CreateMenuBar(), FALSE, FALSE, 0);
+  box = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
 
   /* 加入主区域 */
-  hpaned = gtk_hpaned_new();
-  g_object_set_data(G_OBJECT(hpaned), "position-name",
-                    (gpointer) "main-paned-divide");
-  position = GPOINTER_TO_INT(g_datalist_get_data(&dtset, "main-paned-divide"));
-  gtk_paned_set_position(GTK_PANED(hpaned), position);
-  gtk_box_pack_start(GTK_BOX(box), hpaned, TRUE, TRUE, 0);
-  g_signal_connect(hpaned, "notify::position", G_CALLBACK(PanedDivideChanged),
-                   &dtset);
-  /*/* 加入组成员&附件区域 */
-  vpaned = gtk_vpaned_new();
-  g_object_set_data(G_OBJECT(vpaned), "position-name",
-                    (gpointer) "memberenclosure-paned-divide");
-  position = GPOINTER_TO_INT(
-      g_datalist_get_data(&dtset, "memberenclosure-paned-divide"));
-  gtk_paned_set_position(GTK_PANED(vpaned), position);
-  gtk_paned_pack1(GTK_PANED(hpaned), vpaned, FALSE, TRUE);
-  g_signal_connect(vpaned, "notify::position", G_CALLBACK(PanedDivideChanged),
-                   &dtset);
-  gtk_paned_pack1(GTK_PANED(vpaned), CreateMemberArea(), TRUE, TRUE);
-  gtk_paned_pack2(GTK_PANED(vpaned), CreateFileSendArea(), FALSE, TRUE);
-  /*/* 加入聊天历史记录&输入区域 */
-  vpaned = gtk_vpaned_new();
-  g_object_set_data(G_OBJECT(vpaned), "position-name",
-                    (gpointer) "historyinput-paned-divide");
-  position =
-      GPOINTER_TO_INT(g_datalist_get_data(&dtset, "historyinput-paned-divide"));
-  gtk_paned_set_position(GTK_PANED(vpaned), position);
-  gtk_paned_pack2(GTK_PANED(hpaned), vpaned, TRUE, TRUE);
-  g_signal_connect(vpaned, "notify::position", G_CALLBACK(PanedDivideChanged),
-                   &dtset);
-  gtk_paned_pack1(GTK_PANED(vpaned), CreateHistoryArea(), TRUE, TRUE);
-  gtk_paned_pack2(GTK_PANED(vpaned), DialogBase::CreateInputArea(), FALSE,
+  mainPaned = gtk_paned_new(GTK_ORIENTATION_HORIZONTAL);
+  gtk_paned_set_position(GTK_PANED(mainPaned),
+                         config->GetInt("group_main_paned_divide", 200));
+  gtk_box_pack_start(GTK_BOX(box), mainPaned, TRUE, TRUE, 0);
+  g_signal_connect_swapped(mainPaned, "notify::position",
+                           G_CALLBACK(onUIChanged), this);
+
+  /* 加入组成员&附件区域 */
+  memberEnclosurePaned = gtk_paned_new(GTK_ORIENTATION_VERTICAL);
+  gtk_paned_set_position(
+      GTK_PANED(memberEnclosurePaned),
+      config->GetInt("group_memberenclosure_paned_divide", 100));
+  gtk_paned_pack1(GTK_PANED(mainPaned), memberEnclosurePaned, FALSE, TRUE);
+  g_signal_connect_swapped(memberEnclosurePaned, "notify::position",
+                           G_CALLBACK(onUIChanged), this);
+
+  gtk_paned_pack1(GTK_PANED(memberEnclosurePaned), CreateMemberArea(), TRUE,
+                  TRUE);
+  gtk_paned_pack2(GTK_PANED(memberEnclosurePaned), CreateFileSendArea(), FALSE,
                   TRUE);
 
+  /* 加入聊天历史记录&输入区域 */
+  historyInputPaned = gtk_paned_new(GTK_ORIENTATION_VERTICAL);
+  gtk_paned_set_position(
+      GTK_PANED(historyInputPaned),
+      config->GetInt("group_historyinput_paned_divide", 100));
+  gtk_paned_pack2(GTK_PANED(mainPaned), historyInputPaned, TRUE, TRUE);
+  g_signal_connect_swapped(historyInputPaned, "notify::position",
+                           G_CALLBACK(onUIChanged), this);
+
+  gtk_paned_pack1(GTK_PANED(historyInputPaned), CreateHistoryArea(), TRUE,
+                  TRUE);
+  gtk_paned_pack2(GTK_PANED(historyInputPaned), DialogBase::CreateInputArea(),
+                  FALSE, TRUE);
+
   return box;
 }
 
-/**
- * 创建菜单条.
- * @return 菜单条
- */
-GtkWidget *DialogGroup::CreateMenuBar() {
-  GtkWidget *menubar;
-
-  menubar = gtk_menu_bar_new();
-  gtk_menu_shell_append(GTK_MENU_SHELL(menubar), CreateFileMenu());
-  gtk_menu_shell_append(GTK_MENU_SHELL(menubar), CreateToolMenu());
-  gtk_menu_shell_append(GTK_MENU_SHELL(menubar), CreateHelpMenu());
-
-  return menubar;
-}
-
 /**
  * 创建组成员区域.
  * @return 主窗体
  */
-GtkWidget *DialogGroup::CreateMemberArea() {
+GtkWidget* DialogGroup::CreateMemberArea() {
   GtkWidget *frame, *sw;
-  GtkWidget *widget;
-  GtkTreeModel *model;
+  GtkWidget* widget;
+  GtkTreeModel* model;
 
   frame = gtk_frame_new(_("Member"));
   gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_ETCHED_IN);
@@ -323,8 +309,8 @@ GtkWidget *DialogGroup::CreateMemberArea() {
  * 是否被选中;好友头像;好友昵称;好友数据 \n
  * @return member-model
  */
-GtkTreeModel *DialogGroup::CreateMemberModel() {
-  GtkListStore *model;
+GtkTreeModel* DialogGroup::CreateMemberModel() {
+  GtkListStore* model;
 
   model = gtk_list_store_new(4, G_TYPE_BOOLEAN, GDK_TYPE_PIXBUF, G_TYPE_STRING,
                              G_TYPE_POINTER);
@@ -342,28 +328,27 @@ GtkTreeModel *DialogGroup::CreateMemberModel() {
  * 为群组成员树(member-tree)填充底层数据.
  * @param model member-model
  */
-void DialogGroup::FillMemberModel(GtkTreeModel *model) {
-  GtkIconTheme *theme;
-  GdkPixbuf *pixbuf;
+void DialogGroup::FillMemberModel(GtkTreeModel* model) {
+  GtkIconTheme* theme;
+  GdkPixbuf* pixbuf;
   GtkTreeIter iter;
-  GSList *tlist;
-  PalInfo *pal;
-  char *file;
+  PalInfo* pal;
+  char* file;
 
   theme = gtk_icon_theme_get_default();
+  auto g_cthrd = app->getCoreThread();
   g_cthrd->Lock();
-  tlist = grpinf->member;
-  while (tlist) {
-    pal = (PalInfo *)tlist->data;
+  for (auto ppal : grpinf->getMembers()) {
+    pal = ppal.get();
     file = iptux_erase_filename_suffix(pal->iconfile);
     pixbuf = gtk_icon_theme_load_icon(theme, file, MAX_ICONSIZE,
                                       GtkIconLookupFlags(0), NULL);
     g_free(file);
     gtk_list_store_append(GTK_LIST_STORE(model), &iter);
     gtk_list_store_set(GTK_LIST_STORE(model), &iter, 0, TRUE, 1, pixbuf, 2,
-                       pal->name, 3, pal, -1);
-    if (pixbuf) g_object_unref(pixbuf);
-    tlist = g_slist_next(tlist);
+                       pal->getName().c_str(), 3, pal, -1);
+    if (pixbuf)
+      g_object_unref(pixbuf);
   }
   g_cthrd->Unlock();
 }
@@ -373,11 +358,11 @@ void DialogGroup::FillMemberModel(GtkTreeModel *model) {
  * @param model member-model
  * @return 群组树
  */
-GtkWidget *DialogGroup::CreateMemberTree(GtkTreeModel *model) {
-  GtkWidget *view;
-  GtkTreeSelection *selection;
-  GtkCellRenderer *cell;
-  GtkTreeViewColumn *column;
+GtkWidget* DialogGroup::CreateMemberTree(GtkTreeModel* model) {
+  GtkWidget* view;
+  GtkTreeSelection* selection;
+  GtkCellRenderer* cell;
+  GtkTreeViewColumn* column;
 
   view = gtk_tree_view_new_with_model(model);
   gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(view), TRUE);
@@ -406,131 +391,60 @@ GtkWidget *DialogGroup::CreateMemberTree(GtkTreeModel *model) {
   return view;
 }
 
-/**
- * 创建工具菜单.
- * @return 菜单
- */
-GtkWidget *DialogGroup::CreateToolMenu() {
-  GtkWidget *menushell;
-  GtkWidget *menu, *submenu, *menuitem;
-  GtkTreeModel *model;
-
-  model = GTK_TREE_MODEL(g_datalist_get_data(&mdlset, "member-model"));
-  menushell = gtk_menu_item_new_with_mnemonic(_("_Tools"));
-  menu = gtk_menu_new();
-  gtk_menu_item_set_submenu(GTK_MENU_ITEM(menushell), menu);
-
-  /* 清空历史缓冲区 */
-  NO_OPERATION_C
-  menuitem = gtk_menu_item_new_with_label(_("Clear Buffer"));
-  gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
-  g_signal_connect_swapped(menuitem, "activate", G_CALLBACK(ClearHistoryBuffer),
-                           this);
-
-  /* 群组成员排序 */
-  NO_OPERATION_C
-  menuitem = gtk_menu_item_new_with_label(_("Sort"));
-  gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
-  submenu = gtk_menu_new();
-  gtk_menu_item_set_submenu(GTK_MENU_ITEM(menuitem), submenu);
-  /*/* 按昵称排序 */
-  NO_OPERATION_C
-  menuitem = gtk_radio_menu_item_new_with_label(NULL, _("By Nickname"));
-  g_object_set_data(G_OBJECT(menuitem), "compare-func",
-                    (gpointer)MemberTreeCompareByNameFunc);
-  gtk_menu_shell_append(GTK_MENU_SHELL(submenu), menuitem);
-  g_signal_connect(menuitem, "toggled", G_CALLBACK(SetMemberTreeSortFunc),
-                   model);
-  /*/* 按IP地址排序 */
-  menuitem = gtk_radio_menu_item_new_with_label_from_widget(
-      GTK_RADIO_MENU_ITEM(menuitem), _("By IP"));
-  g_object_set_data(G_OBJECT(menuitem), "compare-func",
-                    (gpointer)MemberTreeCompareByIPFunc);
-  gtk_menu_shell_append(GTK_MENU_SHELL(submenu), menuitem);
-  g_signal_connect(menuitem, "toggled", G_CALLBACK(SetMemberTreeSortFunc),
-                   model);
-  /*/* 分割符 */
-  menuitem = gtk_separator_menu_item_new();
-  gtk_menu_shell_append(GTK_MENU_SHELL(submenu), menuitem);
-  /*/* 升序 */
-  NO_OPERATION_C
-  menuitem = gtk_radio_menu_item_new_with_label(NULL, _("Ascending"));
-  g_object_set_data(G_OBJECT(menuitem), "sort-type",
-                    GINT_TO_POINTER(GTK_SORT_ASCENDING));
-  gtk_menu_shell_append(GTK_MENU_SHELL(submenu), menuitem);
-  g_signal_connect(menuitem, "toggled", G_CALLBACK(SetMemberTreeSortType),
-                   model);
-  /*/* 降序 */
-  menuitem = gtk_radio_menu_item_new_with_label_from_widget(
-      GTK_RADIO_MENU_ITEM(menuitem), _("Descending"));
-  g_object_set_data(G_OBJECT(menuitem), "sort-type",
-                    GINT_TO_POINTER(GTK_SORT_DESCENDING));
-  gtk_menu_shell_append(GTK_MENU_SHELL(submenu), menuitem);
-  g_signal_connect(menuitem, "toggled", G_CALLBACK(SetMemberTreeSortType),
-                   model);
-
-  return menushell;
-}
-
 /**
  * 向选中的好友广播附件消息.
  * @param list 文件链表
  */
-void DialogGroup::BroadcastEnclosureMsg(GSList *list) {
-  SendFile sfile;
-  GtkWidget *widget;
-  GtkTreeModel *model;
+void DialogGroup::BroadcastEnclosureMsg(const vector<FileInfo*>& files) {
+  GtkWidget* widget;
+  GtkTreeModel* model;
   GtkTreeIter iter;
   gboolean active;
-  PalInfo *pal;
-  GSList *plist;
+  PalInfo* pal;
+
+  vector<const PalInfo*> pals;
 
   /* 考察是否有成员 */
   widget = GTK_WIDGET(g_datalist_get_data(&widset, "member-treeview-widget"));
   model = gtk_tree_view_get_model(GTK_TREE_VIEW(widget));
   if (!gtk_tree_model_get_iter_first(model, &iter)) {
-    /**
-     * @note 链表(list)的数据本来应该由(sfile.BcstFileInfoEntry())接手的，
-     * 既然已经没有那个机会了， 当然就只好在这儿手动释放了.
-     */
-    g_slist_foreach(list, GFunc(g_free), NULL);
     return;
   }
 
   /* 向选中的成员发送附件 */
-  plist = NULL;
   do {
     gtk_tree_model_get(model, &iter, 0, &active, 3, &pal, -1);
-    if (active) plist = g_slist_append(plist, pal);
+    if (active) {
+      pals.push_back(pal);
+    }
   } while (gtk_tree_model_iter_next(model, &iter));
-  sfile.BcstFileInfoEntry(plist, list);
-  g_slist_free(plist);
+  app->getCoreThread()->BcstFileInfoEntry(pals, files);
 }
 
 /**
  * 向选中的好友广播文本消息.
  * @param msg 文本消息
  */
-void DialogGroup::BroadcastTextMsg(const gchar *msg) {
-  Command cmd;
-  GtkWidget *widget;
-  GtkTreeModel *model;
+void DialogGroup::BroadcastTextMsg(const gchar* msg) {
+  GtkWidget* widget;
+  GtkTreeModel* model;
   GtkTreeIter iter;
   gboolean active;
   uint32_t opttype;
-  PalInfo *pal;
+  PalInfo* pal;
 
   /* 考察是否有成员 */
   widget = GTK_WIDGET(g_datalist_get_data(&widset, "member-treeview-widget"));
   model = gtk_tree_view_get_model(GTK_TREE_VIEW(widget));
-  if (!gtk_tree_model_get_iter_first(model, &iter)) return;
+  if (!gtk_tree_model_get_iter_first(model, &iter))
+    return;
 
   /* 向选中的成员发送数据 */
   do {
     gtk_tree_model_get(model, &iter, 0, &active, 3, &pal, -1);
     if (active) {
-      if (FLAG_ISSET(pal->flags, 0)) {
-        switch (grpinf->type) {
+      if (pal->isCompatible()) {
+        switch (grpinf->getType()) {
           case GROUP_BELONG_TYPE_BROADCAST:
             opttype = IPTUX_BROADCASTOPT;
             break;
@@ -545,9 +459,10 @@ void DialogGroup::BroadcastTextMsg(const gchar *msg) {
             opttype = IPTUX_REGULAROPT;
             break;
         }
-        cmd.SendUnitMsg(g_cthrd->UdpSockQuote(), pal, opttype, msg);
-      } else
-        cmd.SendGroupMsg(g_cthrd->UdpSockQuote(), pal, msg);
+        app->getCoreThread()->SendUnitMessage(pal->GetKey(), opttype, msg);
+      } else {
+        app->getCoreThread()->SendGroupMessage(pal->GetKey(), msg);
+      }
     }
   } while (gtk_tree_model_iter_next(model, &iter));
 }
@@ -557,7 +472,7 @@ void DialogGroup::BroadcastTextMsg(const gchar *msg) {
  * @param model model
  * @return 菜单
  */
-GtkWidget *DialogGroup::CreatePopupMenu(GtkTreeModel *model) {
+GtkWidget* DialogGroup::CreatePopupMenu(GtkTreeModel* model) {
   GtkWidget *menu, *menuitem;
 
   menu = gtk_menu_new();
@@ -587,33 +502,15 @@ GtkWidget *DialogGroup::CreatePopupMenu(GtkTreeModel *model) {
  * @param b Another GtkTreeIter in model
  * @return 比较值
  */
-gint DialogGroup::MemberTreeCompareByNameFunc(GtkTreeModel *model,
-                                              GtkTreeIter *a, GtkTreeIter *b) {
+gint DialogGroup::MemberTreeCompareByNameFunc(GtkTreeModel* model,
+                                              GtkTreeIter* a,
+                                              GtkTreeIter* b) {
   PalInfo *apal, *bpal;
   gint result;
 
   gtk_tree_model_get(model, a, 3, &apal, -1);
   gtk_tree_model_get(model, b, 3, &bpal, -1);
-  result = strcmp(apal->name, bpal->name);
-
-  return result;
-}
-
-/**
- * 群组成员树(member-tree)按IP排序的比较函数.
- * @param model member-model
- * @param a A GtkTreeIter in model
- * @param b Another GtkTreeIter in model
- * @return 比较值
- */
-gint DialogGroup::MemberTreeCompareByIPFunc(GtkTreeModel *model, GtkTreeIter *a,
-                                            GtkTreeIter *b) {
-  PalInfo *apal, *bpal;
-  gint result;
-
-  gtk_tree_model_get(model, a, 3, &apal, -1);
-  gtk_tree_model_get(model, b, 3, &bpal, -1);
-  result = ntohl(apal->ipv4) - ntohl(bpal->ipv4);
+  result = strcmp(apal->getName().c_str(), bpal->getName().c_str());
 
   return result;
 }
@@ -623,13 +520,14 @@ gint DialogGroup::MemberTreeCompareByIPFunc(GtkTreeModel *model, GtkTreeIter *a,
  * @param menuitem radio-menu-item
  * @param model member-model
  */
-void DialogGroup::SetMemberTreeSortFunc(GtkWidget *menuitem,
-                                        GtkTreeModel *model) {
+void DialogGroup::SetMemberTreeSortFunc(GtkWidget* menuitem,
+                                        GtkTreeModel* model) {
   GtkTreeIterCompareFunc func;
 
-  if (!gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(menuitem))) return;
-  func = (GtkTreeIterCompareFunc)(
-      g_object_get_data(G_OBJECT(menuitem), "compare-func"));
+  if (!gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(menuitem)))
+    return;
+  func = (GtkTreeIterCompareFunc)(g_object_get_data(G_OBJECT(menuitem),
+                                                    "compare-func"));
   gtk_tree_sortable_set_default_sort_func(GTK_TREE_SORTABLE(model), func, NULL,
                                           NULL);
 }
@@ -639,11 +537,12 @@ void DialogGroup::SetMemberTreeSortFunc(GtkWidget *menuitem,
  * @param menuitem radio-menu-item
  * @param model member-model
  */
-void DialogGroup::SetMemberTreeSortType(GtkWidget *menuitem,
-                                        GtkTreeModel *model) {
+void DialogGroup::SetMemberTreeSortType(GtkWidget* menuitem,
+                                        GtkTreeModel* model) {
   GtkSortType type;
 
-  if (!gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(menuitem))) return;
+  if (!gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(menuitem)))
+    return;
   type = (GtkSortType)GPOINTER_TO_INT(
       g_object_get_data(G_OBJECT(menuitem), "sort-type"));
   gtk_tree_sortable_set_sort_column_id(
@@ -656,18 +555,18 @@ void DialogGroup::SetMemberTreeSortType(GtkWidget *menuitem,
  * @param event event
  * @return Gtk+库所需
  */
-gboolean DialogGroup::PopupPickMenu(GtkWidget *treeview,
-                                    GdkEventButton *event) {
-  GtkWidget *menu;
-  GtkTreeModel *model;
+gboolean DialogGroup::PopupPickMenu(GtkWidget* treeview,
+                                    GdkEventButton* event) {
+  GtkWidget* menu;
+  GtkTreeModel* model;
 
-  if (event->button != 3) return FALSE;
+  if (!gdk_event_triggers_context_menu((GdkEvent*)event)) {
+    return FALSE;
+  }
   model = gtk_tree_view_get_model(GTK_TREE_VIEW(treeview));
   menu = CreatePopupMenu(model);
   gtk_widget_show_all(menu);
-  gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL, event->button,
-                 event->time);
-
+  gtk_menu_popup_at_pointer(GTK_MENU(menu), NULL);
   return TRUE;
 }
 
@@ -677,39 +576,39 @@ gboolean DialogGroup::PopupPickMenu(GtkWidget *treeview,
  * @param path path
  * @param column column
  */
-void DialogGroup::MembertreeItemActivated(GtkWidget *treeview,
-                                          GtkTreePath *path,
-                                          GtkTreeViewColumn *column,
-                                          DialogGroup *self) {
-  GtkTreeModel *model;
+void DialogGroup::MembertreeItemActivated(GtkWidget* treeview,
+                                          GtkTreePath* path,
+                                          GtkTreeViewColumn*,
+                                          DialogGroup* self) {
+  GtkTreeModel* model;
   GtkTreeIter iter;
-  PalInfo *pal;
-  GroupInfo *grpinf;
+  PalInfo* pal;
+  GroupInfo* grpinf;
 
   model = gtk_tree_view_get_model(GTK_TREE_VIEW(treeview));
   gtk_tree_model_get_iter(model, &iter, path);
   gtk_tree_model_get(model, &iter, 3, &pal, -1);
-  if ((grpinf = g_cthrd->GetPalRegularItem(pal))) {
-    if ((grpinf->dialog))
-      gtk_window_present(GTK_WINDOW(grpinf->dialog));
+  if ((grpinf = self->app->getCoreThread()->GetPalRegularItem(pal))) {
+    if ((grpinf->getDialog()))
+      gtk_window_present(GTK_WINDOW(grpinf->getDialog()));
     else
-      DialogPeer::PeerDialogEntry(self->config, grpinf, self->progdt);
+      DialogPeer::PeerDialogEntry(self->app, grpinf);
   }
 }
 
 bool DialogGroup::SendTextMsg() {
-  GtkWidget *textview;
-  GtkTextBuffer *buffer;
+  GtkWidget* textview;
+  GtkTextBuffer* buffer;
   GtkTextIter start, end;
-  MsgPara msgpara;
-  gchar *msg;
+  gchar* msg;
 
   /* 考察缓冲区内是否存在数据 */
   textview = GTK_WIDGET(g_datalist_get_data(&widset, "input-textview-widget"));
   gtk_widget_grab_focus(textview);  //为下一次任务做准备
   buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(textview));
   gtk_text_buffer_get_bounds(buffer, &start, &end);
-  if (gtk_text_iter_equal(&start, &end)) return false;
+  if (gtk_text_iter_equal(&start, &end))
+    return false;
 
   /* 获取数据并发送 */
   msg = gtk_text_buffer_get_text(buffer, &start, &end, FALSE);
@@ -717,9 +616,9 @@ bool DialogGroup::SendTextMsg() {
   FeedbackMsg(msg);
   BroadcastTextMsg(msg);
 
-  msgpara.stype = MESSAGE_SOURCE_TYPE_SELF;
-  msgpara.pal = NULL;
-  g_lgsys->CommunicateLog(&msgpara, "[STRING]%s", msg);
+  MsgPara msgpara(this->app->getMe());
+  msgpara.stype = MessageSourceType::SELF;
+  app->getLogSystem()->communicateLog(&msgpara, "[STRING]%s", msg);
   g_free(msg);
 
   return true;
@@ -729,7 +628,7 @@ bool DialogGroup::SendTextMsg() {
  * 发送消息.
  * @param dlggrp 对话框类
  */
-void DialogGroup::SendMessage(DialogGroup *dlggrp) {
+void DialogGroup::SendMessage(DialogGroup* dlggrp) {
   dlggrp->SendEnclosureMsg();
   dlggrp->SendTextMsg();
 }
@@ -738,25 +637,84 @@ void DialogGroup::SendMessage(DialogGroup *dlggrp) {
  * @return plist 获取待发送成员列表
  * 调用该函数后须free plist
  */
-GSList *DialogGroup::GetSelPal() {
-  GtkWidget *widget;
-  GtkTreeModel *model;
+GSList* DialogGroup::GetSelPal() {
+  GtkWidget* widget;
+  GtkTreeModel* model;
   GtkTreeIter iter;
   gboolean active;
-  PalInfo *pal;
-  GSList *plist;
+  PalInfo* pal;
+  GSList* plist;
 
   /* 考察是否有成员 */
   widget = GTK_WIDGET(g_datalist_get_data(&widset, "member-treeview-widget"));
   model = gtk_tree_view_get_model(GTK_TREE_VIEW(widget));
-  if (!gtk_tree_model_get_iter_first(model, &iter)) return NULL;
+  if (!gtk_tree_model_get_iter_first(model, &iter))
+    return NULL;
 
   plist = NULL;
   do {
     gtk_tree_model_get(model, &iter, 0, &active, 3, &pal, -1);
-    if (active) plist = g_slist_append(plist, pal);
+    if (active)
+      plist = g_slist_append(plist, pal);
   } while (gtk_tree_model_iter_next(model, &iter));
   return plist;
 }
 
+void DialogGroup::onUIChanged(DialogGroup& self) {
+  self.config->SetInt("group_main_paned_divide",
+                      gtk_paned_get_position(GTK_PANED(self.mainPaned)));
+  self.config->SetInt(
+      "group_memberenclosure_paned_divide",
+      gtk_paned_get_position(GTK_PANED(self.memberEnclosurePaned)));
+  self.config->SetInt(
+      "group_historyinput_paned_divide",
+      gtk_paned_get_position(GTK_PANED(self.historyInputPaned)));
+  self.config->Save();
+}
+
+void DialogGroup::onSortBy(GSimpleAction* action,
+                           GVariant* value,
+                           DialogGroup& self) {
+  string sortBy = g_variant_get_string(value, nullptr);
+
+  PalTreeModelSortKey key;
+
+  if (sortBy == "nickname") {
+    key = PalTreeModelSortKey::NICKNAME;
+  } else if (sortBy == "ip") {
+    key = PalTreeModelSortKey::IP;
+  } else {
+    LOG_WARN("unknown sort by: %s", sortBy.c_str());
+    return;
+  }
+
+  auto model =
+      GTK_TREE_MODEL(g_datalist_get_data(&self.mdlset, "member-model"));
+  palTreeModelSetSortKey(model, key);
+  g_simple_action_set_state(action, value);
+}
+
+void DialogGroup::onSortType(GSimpleAction* action,
+                             GVariant* value,
+                             DialogGroup& self) {
+  string sortType = g_variant_get_string(value, nullptr);
+
+  GtkSortType type;
+
+  if (sortType == "ascending") {
+    type = GTK_SORT_ASCENDING;
+  } else if (sortType == "descending") {
+    type = GTK_SORT_DESCENDING;
+  } else {
+    LOG_WARN("unknown sorttype: %s", sortType.c_str());
+    return;
+  }
+
+  auto model =
+      GTK_TREE_MODEL(g_datalist_get_data(&self.mdlset, "member-model"));
+  gtk_tree_sortable_set_sort_column_id(
+      GTK_TREE_SORTABLE(model), GTK_TREE_SORTABLE_DEFAULT_SORT_COLUMN_ID, type);
+  g_simple_action_set_state(action, value);
+}
+
 }  // namespace iptux
